package com.magnatune.eyecreate.companionformagnatune.model;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v4.content.LocalBroadcastManager;

import com.magnatune.eyecreate.companionformagnatune.MagnatuneCompainionApplication;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import io.realm.Realm;
import io.realm.RealmList;
import retrofit.Response;

public class MagnatuneDBManager {

    public static final String ACTION_DB_UPDATE_DONE = "dbupdatedone";
    private static final String DB_CHECK_PREFS = "dbcheckprefs";
    private static final String SP_KEY_DB_HASH = "dbhash";

    public static void updateDB() {
        //This method should be called on another thread
        //Check if DB should update
        SharedPreferences prefs = MagnatuneCompainionApplication.getApplication().getSharedPreferences(DB_CHECK_PREFS, Context.MODE_PRIVATE);
        boolean shouldUpdate = false;
        if(prefs.contains(SP_KEY_DB_HASH)) {
            String hash = "";
            try {
                hash = MagnatuneCompainionApplication.getDataApi().getUpdateChecksum().execute().body();
            } catch (IOException e) {
                e.printStackTrace();
            }
            if(!hash.equals(prefs.getString(SP_KEY_DB_HASH,""))) {
                shouldUpdate = true;
                prefs.edit().putString(SP_KEY_DB_HASH,hash).apply();
            }
        } else {
            try {
                shouldUpdate = true;
                prefs.edit().putString(SP_KEY_DB_HASH,MagnatuneCompainionApplication.getDataApi().getUpdateChecksum().execute().body()).apply();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if(shouldUpdate) {
            //Update DB
            try {
                Response<AlbumResponse> response = MagnatuneCompainionApplication.getDataApi().getAlbumList().execute();
                Realm db = getAlbumDB();
                db.beginTransaction();
                for(com.magnatune.eyecreate.companionformagnatune.model.xml.Album album:response.body().albums) {
                    db.copyToRealmOrUpdate(xmlAlbumToDBAlbum(album));
                }
                db.commitTransaction();
                db.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //Broadcast Refresh done.
        LocalBroadcastManager.getInstance(MagnatuneCompainionApplication.getApplication()).sendBroadcast(new Intent(ACTION_DB_UPDATE_DONE));
    }

    public static Realm getAlbumDB() {
        //Remember to close each instance when done.
        return Realm.getInstance(MagnatuneCompainionApplication.getAlbumDBConfig());
    }

    public static RealmList<MGenre> genreCSVStringToMGenreList(String xmlgenres) {
        List<String> genres = Arrays.asList(xmlgenres.split(","));
        RealmList<MGenre> DBgenres = new RealmList<>();
        for(String genre:genres) {
            MGenre mGenre = new MGenre();
            mGenre.setGenre_name(genre);
            DBgenres.add(mGenre);
        }
        return DBgenres;
    }

    public static Album xmlAlbumToDBAlbum(com.magnatune.eyecreate.companionformagnatune.model.xml.Album album) {
        Album entry = new Album();
        entry.setAlbumsku(album.albumsku);
        entry.setAlbumname(album.albumname);
        entry.setArtist(album.artist);
        entry.setArtistdesc(album.artistdesc);
        entry.setAlbum_notes(album.album_notes);
        entry.setCover_small(album.cover_small);
        entry.setCover_large(album.cover_small.replace("200", "800"));
        entry.setArtistphoto(album.artistphoto);
        entry.setYear(album.year);
        entry.setHomepage(album.home);
        entry.setMp3genre(album.mp3genre);
        entry.setLaunchdate(album.launchdate);
        entry.setGenres(genreCSVStringToMGenreList(album.magnatunegenres));
        RealmList<Track> tracks = new RealmList<>();
        for(com.magnatune.eyecreate.companionformagnatune.model.xml.Track track:album.tracks) {
            tracks.add(xmlTrackToDBTrack(track));
        }
        entry.setTracks(tracks);
        return entry;
    }

    public static Track xmlTrackToDBTrack(com.magnatune.eyecreate.companionformagnatune.model.xml.Track track) {
        Track entry = new Track();
        entry.setIsrc(track.isrc);
        entry.setTrackname(track.trackname);
        entry.setArtist(track.artist);
        entry.setAlbumname(track.albumname);
        entry.setTracknumber(track.tracknum);
        entry.setYear(track.year);
        entry.setMp3genre(track.mp3genre);
        entry.setGenres(genreCSVStringToMGenreList(track.magnatunegenres));
        entry.setLicense(track.license);
        entry.setSeconds(track.seconds);
        entry.setUrl(track.url);
        entry.setMp3_low_quality(track.mp3lofi);
        entry.setOgg_url(track.oggurl);
        return entry;
    }
}
